import { requestUrl } from "obsidian";
import {
	Contributor,
	NextData,
	Ref,
	Book as ApiBook,
	SearchResult,
} from "./types";
import { Author, Book, Series } from "../../types";

export function createBookFromNextData(nextData: NextData, ref: Ref): Book {
	const apolloState = nextData.props.pageProps.apolloState;
	const bookData = apolloState[ref.__ref] as ApiBook;

	const contributorEdges = [
		bookData.primaryContributorEdge,
		...bookData.secondaryContributorEdges,
	];
	const authors = contributorEdges
		.filter((edge) => edge.role === "Author")
		.map<Contributor>((edge) => apolloState[edge.node.__ref])
		.map<Author>((contributor) => ({
			id: contributor.id,
			legacyId: contributor.legacyId,
			name: contributor.name,
			description: contributor.description,
		}));

	let series: Series | null = null;
	if (bookData.bookSeries.length > 0) {
		const bookSeries = bookData.bookSeries[0];
		const seriesData = apolloState[bookSeries.series.__ref];
		series = {
			title: seriesData.title,
			position: parseInt(bookSeries.userPosition, 10),
		};
	}

	return {
		title: bookData.title,
		description: bookData.description,
		authors,
		series,
		publisher: bookData.details.publisher,
		publishedAt: new Date(bookData.details.publicationTime),
		genres: bookData.bookGenres.map((genre) => genre.genre.name),
		coverImageUrl: bookData.imageUrl,
		pageLength: bookData.details.numPages,
		isbn: bookData.details.isbn,
		isbn13: bookData.details.isbn13,
	};
}

async function getNextData(legacyId: number): Promise<NextData> {
	const url = "https://www.goodreads.com/book/show/" + legacyId;
	const res = await requestUrl({ url });
	const doc = new DOMParser().parseFromString(res.text, "text/html");
	let nextDataRaw = doc.getElementById("__NEXT_DATA__")?.textContent;
	if (typeof nextDataRaw !== "string") {
		throw new Error("Unable to find next data script in the document.");
	}
	const nextData = JSON.parse(nextDataRaw) as NextData;

	return nextData;
}

export async function getBookByLegacyId(legacyId: number): Promise<Book> {
	const nextData = await getNextData(legacyId);
	const bookRef = nextData.props.pageProps.apolloState.ROOT_QUERY[
		`getBookByLegacyId({"legacyId":"${legacyId}"})`
	] as Ref | undefined;
	if (bookRef === undefined) {
		throw new Error("Could not find reference for book.");
	}

	return createBookFromNextData(nextData, bookRef);
}

export async function searchBooks(q: string): Promise<SearchResult[]> {
	const url = "https://www.goodreads.com/search?q=" + encodeURIComponent(q);
	const res = await requestUrl({ url });
	const doc = new DOMParser().parseFromString(res.text, "text/html");
	const bookElements = doc.querySelectorAll(
		"table.tableList tr[itemtype='http://schema.org/Book']"
	);

	const searchResults: SearchResult[] = [];

	bookElements.forEach((el) => {
		const legacyId = parseInt(
			el.querySelector("div.u-anchorTarget")?.id ?? "",
			10
		);
		const title = el.querySelector("a.bookTitle")?.textContent?.trim();
		const authors = Array.from(el.querySelectorAll("a.authorName"))
			.map((a) => a.textContent?.trim() || "")
			.join(", ");
		const coverImageUrl = el
			.querySelector("img.bookCover")
			?.getAttribute("src");
		const avgRating = parseFloat(
			el
				.querySelector("span.minirating")
				?.textContent?.match(/(\d+\.\d+) avg rating/)?.[1] ?? "0"
		);
		const ratingCount = parseInt(
			el
				.querySelector("span.minirating")
				?.textContent?.match(/(\d[\d,]*) ratings/)?.[1] ?? "0",
			10
		);
		const publicationYear = parseInt(
			el
				.querySelector("span.greyText")
				?.textContent?.match(/published (\d{4})/)?.[1] ?? "0",
			10
		);
		const editionCount = parseInt(
			el
				.querySelector("span.greyText")
				?.textContent?.match(/(\d+) editions/)?.[1] ?? "0",
			10
		);

		if (
			!isNaN(legacyId) &&
			title &&
			authors &&
			!isNaN(avgRating) &&
			!isNaN(ratingCount) &&
			!isNaN(publicationYear) &&
			!isNaN(editionCount) &&
			coverImageUrl
		) {
			searchResults.push({
				legacyId,
				title,
				authors: authors.split(", "),
				avgRating,
				ratingCount,
				publicationYear,
				editionCount,
				coverImageUrl,
			});
		}
	});

	return searchResults;
}
