import BookTrackerPlugin from "@src/main";
import { App, PluginSettingTab, Setting } from "obsidian";
import { FileSuggest } from "./suggesters/file";
import { FolderSuggest } from "./suggesters/folder";

export interface BookTrackerPluginSettings {
	templateFile: string;
	tbrDirectory: string;
	fileNameFormat: string;
	downloadCovers: boolean;
	coverDirectory: string;
	groupCoversByFirstLetter: boolean;
	overwriteExistingCovers: boolean;
}

export const DEFAULT_SETTINGS: BookTrackerPluginSettings = {
	templateFile: "",
	tbrDirectory: "books/tbr",
	fileNameFormat: "{{title}} - {{authors}}",
	downloadCovers: false,
	coverDirectory: "images/covers",
	groupCoversByFirstLetter: true,
	overwriteExistingCovers: false,
};

export class BookTrackerSettingTab extends PluginSettingTab {
	constructor(app: App, private plugin: BookTrackerPlugin) {
		super(app, plugin);
	}

	heading(text: string): void {
		const header = document.createDocumentFragment();
		header.createEl("h2", { text });
		new Setting(this.containerEl).setHeading().setName(text);
	}

	display(): void {
		this.containerEl.empty();
		this.containerEl.classList.add("obt-settings");

		this.heading("Book Creation Settings");
		this.templateFileSetting();
		this.tbrDirectorySetting();
		this.fileNameFormatSetting();

		this.heading("Cover Download Settings");
		this.downloadCoversSetting();
		this.coverDirectorySetting();
		this.groupCoversByFirstLetterSetting();
		this.overwriteExistingCoversSetting();
	}

	overwriteExistingCoversSetting() {
		return new Setting(this.containerEl)
			.setName("Overwrite Existing Covers")
			.setDesc("Overwrite existing book covers when downloading new ones")
			.addToggle((cb) => {
				cb.setValue(
					this.plugin.settings.overwriteExistingCovers
				).onChange(async (value) => {
					this.plugin.settings.overwriteExistingCovers = value;
					await this.plugin.saveSettings();
				});
			});
	}

	groupCoversByFirstLetterSetting() {
		return new Setting(this.containerEl)
			.setName("Group Covers by First Letter")
			.setDesc(
				"Organize downloaded book covers into subdirectories based on the first letter of the book title"
			)
			.addToggle((cb) => {
				cb.setValue(
					this.plugin.settings.groupCoversByFirstLetter
				).onChange(async (value) => {
					this.plugin.settings.groupCoversByFirstLetter = value;
					await this.plugin.saveSettings();
				});
			});
	}

	coverDirectorySetting() {
		return new Setting(this.containerEl)
			.setName("Cover Directory")
			.setDesc(
				"Select the directory where downloaded book covers will be stored"
			)
			.addSearch((cb) => {
				try {
					new FolderSuggest(this.app, cb.inputEl);
				} catch {
					// If the suggest fails, we can just ignore it.
					// This might happen if the plugin is not fully loaded yet.
				}

				cb.setPlaceholder("images/covers")
					.setValue(this.plugin.settings.coverDirectory)
					.onChange(async (value) => {
						this.plugin.settings.coverDirectory = value;
						await this.plugin.saveSettings();
					});
			});
	}

	downloadCoversSetting() {
		return new Setting(this.containerEl)
			.setName("Download Covers")
			.setDesc(
				"Automatically download book covers when creating new entries"
			)
			.addToggle((cb) => {
				cb.setValue(this.plugin.settings.downloadCovers).onChange(
					async (value) => {
						this.plugin.settings.downloadCovers = value;
						await this.plugin.saveSettings();
					}
				);
			});
	}

	fileNameFormatSetting() {
		const fileNameFormatDesc = document.createDocumentFragment();
		fileNameFormatDesc.createDiv({
			text: "Format for the file name of new book entries.",
		});
		fileNameFormatDesc.createDiv({
			text: "Use {{title}} and {{authors}} as placeholders.",
		});

		new Setting(this.containerEl)
			.setName("File Name Format")
			.setDesc(fileNameFormatDesc)
			.addText((cb) => {
				cb.setPlaceholder("{{title}} - {{authors}}")
					.setValue(this.plugin.settings.fileNameFormat)
					.onChange(async (value) => {
						this.plugin.settings.fileNameFormat = value;
						await this.plugin.saveSettings();
					});
			});
	}

	tbrDirectorySetting() {
		return new Setting(this.containerEl)
			.setName("To Be Read Directory")
			.setDesc(
				"Select the directory where new book entries will be created"
			)
			.addSearch((cb) => {
				try {
					new FolderSuggest(this.app, cb.inputEl);
				} catch {
					// If the suggest fails, we can just ignore it.
					// This might happen if the plugin is not fully loaded yet.
				}
				const { containerEl } = this;

				cb.setPlaceholder("books/tbr")
					.setValue(this.plugin.settings.tbrDirectory)
					.onChange(async (value) => {
						this.plugin.settings.tbrDirectory = value;
						await this.plugin.saveSettings();
					});
			});
	}

	templateFileSetting() {
		return new Setting(this.containerEl)
			.setName("Template File")
			.setDesc("Select the template file to use for new book entries")
			.addSearch((cb) => {
				try {
					new FileSuggest(this.app, cb.inputEl);
				} catch {
					// If the suggest fails, we can just ignore it.
					// This might happen if the plugin is not fully loaded yet.
				}

				cb.setPlaceholder("templates/book-template")
					.setValue(this.plugin.settings.templateFile)
					.onChange(async (value) => {
						this.plugin.settings.templateFile = value;
						await this.plugin.saveSettings();
					});
			});
	}
}
